@@ -1,6 +1,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#define IMAGER_NO_CONTEXT
 #include "imager.h"
 
 /*
@@ -9,7 +10,6 @@
 
 struct i_bitmap*
 btm_new(i_img_dim xsize,i_img_dim ysize) {
-  i_img_dim i;
   size_t bytes;
   struct i_bitmap *btm;
   btm=(struct i_bitmap*)mymalloc(sizeof(struct i_bitmap)); /* checked 4jul05 tonyc */
@@ -130,8 +130,9 @@ llist_push(struct llist *l,const void *data) {
     }
   }
   /*   fprintf(stderr,"0x%08X\n",l->t); */
-  if (llist_llink_push(l,l->t,data)) { 
-    i_fatal(3, "out of memory\n");
+  if (llist_llink_push(l,l->t,data)) {
+    dIMCTX;
+    im_fatal(aIMCTX, 3, "out of memory\n");
   }
 }
 
